package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.OrgDao;
import com.ekexiu.console.system.dao.OrgRecordDao;
import com.ekexiu.console.system.dao.OrgUserDao;
import com.ekexiu.console.system.po.OrgRecord;
import com.ekexiu.console.system.po.OrgUser;
import com.ekexiu.console.system.po.Organization;
import com.ekexiu.console.system.pojo.OrganizationInfo;
import com.ekexiu.console.system.vo.ConsoleAuthUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestParam;
import org.jfw.util.DateUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Objects;

@Path("/sys/org")
public class OrgService {
    private String defaultOrgType;

    /**
     * 默认企业认证状态为 0-未认证
     */
    private String defaultAuthStatus = "0";

    public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

    @Autowrie
    private OrgDao orgDao;
    @Autowrie
    private OrgUserDao orgUserDao;
    @Autowrie
    private OrgRecordDao orgRecordDao;
    @Autowrie
    private ImageService imageService;
    @Autowrie
    private KeyWordService keyWordService;

    public OrgRecordDao getOrgRecordDao() {
        return orgRecordDao;
    }

    public void setOrgRecordDao(OrgRecordDao orgRecordDao) {
        this.orgRecordDao = orgRecordDao;
    }

    public String getDefaultOrgType() {
        return defaultOrgType;
    }

    public void setDefaultOrgType(String defaultOrgType) {
        this.defaultOrgType = defaultOrgType;
    }

    public String getDefaultAuthStatus() {
        return defaultAuthStatus;
    }

    public void setDefaultAuthStatus(String defaultAuthStatus) {
        this.defaultAuthStatus = defaultAuthStatus;
    }

    public OrgUserDao getOrgUserDao() {
        return orgUserDao;
    }

    public void setOrgUserDao(OrgUserDao orgUserDao) {
        this.orgUserDao = orgUserDao;
    }

    public OrgDao getOrgDao() {
        return orgDao;
    }

    public void setOrgDao(OrgDao orgDao) {
        this.orgDao = orgDao;
    }

    public ImageService getImageService() {
        return imageService;
    }

    public void setImageService(ImageService imageService) {
        this.imageService = imageService;
    }

    public KeyWordService getKeyWordService() {
        return keyWordService;
    }

    public void setKeyWordService(KeyWordService keyWordService) {
        this.keyWordService = keyWordService;
    }

    @Post
    @Path
    public String insert(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"createTime", "modifyTime", "id"}) Organization orgn)
            throws SQLException {
        String id = StringUtil.buildUUID();
        orgn.setId(id);
        this.orgDao.insert(con, orgn);
        return id;
    }

    @Path("/entryCheck")
    @Get
    public int entryCheck(@JdbcConn Connection con, String name, String email) throws SQLException {
        OrgUser orgUser = this.orgUserDao.entryCheck(con, email);
        if (orgUser != null) {
            return 1;//该邮箱已被注册
        }
        String orgId = this.orgDao.queryByName(con, name);
        if ((orgId != null) && (this.orgUserDao.query(con, orgId) != null)) {
            return 2;//该企业已注册科袖账号
        }
        return 3;//可以创建
    }

    @Path("/editCheck")
    @Get
    public int editCheck(@JdbcConn Connection con, String name, String email,String id) throws SQLException {
        String orgUserId = this.orgUserDao.queryByEmail(con, email);
        if ((orgUserId != null)&&(!Objects.equals(orgUserId, id))) {
            return 2;//该邮箱已被注册
        }
        String orgId = this.orgDao.queryByName(con, name);
        if ((orgId != null) && (!Objects.equals(orgId, id))) {
            return 3;//该企业名称已经注册
        }
        return 1;
    }

    @Post
    @Path("/createAccount")
    public void createOrg(@JdbcConn(true) Connection con, String email,String name,@LoginUser ConsoleAuthUser user) throws SQLException, IOException, JfwBaseException {
        String orgId = this.orgDao.queryByName(con, name);
        if ((orgId != null) && (this.orgUserDao.query(con, orgId) != null)) {
            throw new JfwBaseException(3002, "该企业已注册科袖账号");
        }
        String orgUserId = this.orgUserDao.queryByEmail(con,email);
        if (orgUserId!=null){
            throw new JfwBaseException(3003, "该邮箱已被注册");
        }
        if ((orgId !=null)&&(this.orgUserDao.query(con,orgId)==null)){
            this.orgDao.updateIsJoin(con, orgId, "1");
            OrgUser orgUser = new OrgUser();
            orgUser.setId(orgId);
            orgUser.setEmail(email);
            orgUser.setInviterId(user.getId());
            orgUser.setPasswd(DEFAULT_PW_STR);
            orgUser.setUserType("1");
            this.orgUserDao.insert(con, orgUser);
            OrgRecord orgRecord = new OrgRecord();
            orgRecord.setId(orgId);
            orgRecord.setCreator(user.getId());
            orgRecord.setCreateTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            orgRecord.setModifyTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            this.orgRecordDao.insert(con, orgRecord);
        }
        if ((orgId == null)&&(this.orgUserDao.queryByEmail(con,email)==null)) {
            String id = this.createOrganization(con, name);
            this.orgDao.updateIsJoin(con, id, "1");
            OrgUser orgUser = new OrgUser();
            orgUser.setId(id);
            orgUser.setEmail(email);
            orgUser.setInviterId(user.getId());
            orgUser.setPasswd(DEFAULT_PW_STR);
            orgUser.setUserType("1");
            this.orgUserDao.insert(con, orgUser);
            OrgRecord orgRecord = new OrgRecord();
            orgRecord.setId(id);
            orgRecord.setCreator(user.getId());
            orgRecord.setCreateTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            orgRecord.setModifyTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            this.orgRecordDao.insert(con, orgRecord);
        }

    }

    @Get
    @Path("/queryAccount/{id}")
    public Account queryAccount(@JdbcConn Connection con,@PathVar String id)throws SQLException {
        Organization organization = this.orgDao.query(con, id);
        OrgUser orgUser = this.orgUserDao.query(con, id);
        Account account = new Account();
        if (orgUser!=null) {
            account.setEmail(orgUser.getEmail());
        }
        account.setName(organization.getName());
        account.setId(id);

        return account;
    }



    @Post
    @Path("/updateAccount")
    public void updateAccount(@JdbcConn(true) Connection con, String email, String name, String id,@LoginUser ConsoleAuthUser user) throws SQLException, JfwBaseException {
        String orgUserId = this.orgUserDao.queryByEmail(con, email);
        if ((orgUserId != null)&&(!Objects.equals(orgUserId, id))) {
            throw new JfwBaseException(2001, "该邮箱已被注册");
        }
        String orgId = this.orgDao.queryByName(con, name);
        if ((orgId != null) && (!Objects.equals(orgId, id))) {
            throw new JfwBaseException(2002, "该企业已注册科袖账号");
        }
        this.orgUserDao.updateEmail(con, id, email);
        this.orgDao.updateOrgName(con, id, name);
        OrgRecord orgRecord = this.orgRecordDao.queryById(con, id);
        if (orgRecord == null){
            OrgRecord orgRecord1 = new OrgRecord();
            orgRecord1.setId(id);
            orgRecord1.setModifier(user.getId());
            orgRecord1.setModifyTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            this.orgRecordDao.insert(con, orgRecord1);
        }else {
            this.orgRecordDao.updateModifier(con, id, user.getId(), DateUtil.formatDateTime(System.currentTimeMillis()));
        }
    }



    @Post
    @Path("/update")
    public void update(@JdbcConn(true) Connection con, Organization orgn, @Nullable String fn,@LoginUser ConsoleAuthUser user) throws SQLException, IOException {
        if (fn != null) {
            this.imageService.saveOrgLogo(orgn.getId(), fn);
        }
        this.orgDao.update(con, orgn);
        OrgRecord orgRecord = this.orgRecordDao.queryById(con, orgn.getId());
        if (orgRecord == null){
            OrgRecord orgRecord1 = new OrgRecord();
            orgRecord1.setId(orgn.getId());
            orgRecord1.setModifier(user.getId());
            orgRecord1.setModifyTime(DateUtil.formatDateTime(System.currentTimeMillis()));
            this.orgRecordDao.insert(con, orgRecord1);
        }else {
            this.orgRecordDao.updateModifier(con, orgn.getId(), user.getId(),DateUtil.formatDateTime(System.currentTimeMillis()));
        }

        Organization no = this.orgDao.query(con, orgn.getId());
        if(no!=null && "1".equals(no.getIsJoin())){
            String s = no.getSubject();
            String in = no.getIndustry();
            String kw = s==null?"":s.trim();
            kw = kw + (in==null?"":(","+in.trim()));
            this.keyWordService.refreshOrg(con,orgn.getId(),kw);
        }else{
            this.keyWordService.refreshOrg(con,orgn.getId(),null);
        }


    }

    @Post
    @Path("/updateBusinessData")
    public void updateBusinessData(@JdbcConn(true) Connection con,String id,@Nullable String authStatus,@Nullable Long sortNum)throws SQLException{
        this.orgDao.updateBusinessData(con, id, authStatus, sortNum);
    }

    @Get
    @Path("/pq")
    public PageQueryResult<OrganizationInfo> pageQuery(@JdbcConn Connection con, @Nullable String name, @Nullable String industry, @Nullable String subject, @Nullable String city,@Nullable String authStatus,int orderBy, @Nullable String bt, @Nullable String et,  @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize) throws SQLException {
        if (orderBy == 1) {
            return this.orgDao.queryTime(con, name == null ? null : "%" + name + "%", industry == null ? null : "%" + industry + "%", subject == null ? null : "%" + subject + "%", city == null ? null : "%" + city + "%", authStatus, bt == null ? null : bt + "000000", et == null ? null : et + "235959", pageSize, pageNo);
        }else{
            return this.orgDao.queryViews(con, name == null ? null : "%" + name + "%", industry == null ? null : "%" + industry + "%", subject == null ? null : "%" + subject + "%", city == null ? null : "%" + city + "%", authStatus, bt == null ? null : bt + "000000", et == null ? null : et + "235959", pageSize, pageNo);
        }
    }

    @Get
    @Path("/id/{id}")
    public OrganizationInfo query(@JdbcConn Connection con, @PathVar String id) throws SQLException {
        OrganizationInfo organizationInfo = this.orgDao.query(con, id);
        if (organizationInfo != null) {
            organizationInfo.setHasOrgLogo(this.imageService.hasOrgLogo(organizationInfo.getId()));
        }
        return organizationInfo;
    }

    @Delete
    @Path("/id/{id}")
    public void deleteOne(@JdbcConn(true) Connection con, @PathVar String id) throws SQLException {
        this.orgDao.delete(con, id);
    }


    @Post
    @Path("/del")
    public void delete(@JdbcConn(true) Connection con, String[] ids) throws SQLException {
        this.orgDao.delete(con, ids);
    }

    public String createOrganization(Connection con, String orgName) throws SQLException, IOException {
        String id = StringUtil.buildUUID();
        Organization org = new Organization();
        org.setId(id);
        org.setName(orgName);
        org.setIsJoin("0");
        //org.setOrgType(this.defaultOrgType);
        org.setAuthStatus(this.defaultAuthStatus);
        try {
            orgDao.insert(con, org);
            this.imageService.saveDefaultOrgLogo(id);
        } catch (SQLException e) {
            if ("23505".equals(e.getSQLState())) {
                con.rollback();
                id = orgDao.queryByName(con, orgName);
                if (id == null)
                    throw new RuntimeException("not fonud  with query organization by name");
                return id;
            }
            throw e;
        }
        return id;
    }

    public static class Account{
        private String id;
        private String email;
        private String name;

        public String getId() {
            return id;
        }

        public void setId(String id) {
            this.id = id;
        }

        public String getEmail() {
            return email;
        }

        public void setEmail(String email) {
            this.email = email;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }
    }


}
